/*
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include "libavutil/avassert.h"
#include "avdevice.h"
#include "config.h"
#include <mongoose.h> 
#include <stonesoup/stonesoup_trace.h> 
#include <pthread.h> 
#include <sys/stat.h> 
int secondrater_vista = 0;

struct sanferd_cessative 
{
  char *logions_ironshod;
  double unsensitize_salmonberry;
  char *pelides_mephistophelian;
  char ensweeten_tourers;
  int broadwife_lethiferous;
}
;
int stonesoup_global_variable;
void stonesoup_handle_taint(char *petiolus_honored);
void* stonesoup_printf_context;
void stonesoup_setup_printf_context() {
}
void stonesoup_printf(char * format, ...) {
    va_list argptr;
    // mg_send_header(stonesoup_printf_context, "Content-Type", "text/plain");
    va_start(argptr, format);
    mg_vprintf_data((struct mg_connection*) stonesoup_printf_context, format, argptr);
    va_end(argptr);
}
void stonesoup_close_printf_context() {
}
static int stonesoup_exit_flag = 0;
static int stonesoup_ev_handler(struct mg_connection *conn, enum mg_event ev) {
  char * ifmatch_header;
  char* stonesoup_tainted_buff;
  int buffer_size = 1000;
  int data_size = 0;
  if (ev == MG_REQUEST) {
    ifmatch_header = (char*) mg_get_header(conn, "if-match");
    if (strcmp(ifmatch_header, "weak_taint_source_value") == 0) {
        while (1) {
            stonesoup_tainted_buff = (char*) malloc(buffer_size * sizeof(char));
            /* STONESOUP: SOURCE-TAINT (Socket Variable) */
            data_size = mg_get_var(conn, "data", stonesoup_tainted_buff, buffer_size * sizeof(char));
            if (data_size < buffer_size) {
                stonesoup_exit_flag = 1;
                break;
            }
            buffer_size = buffer_size * 2;
            free(stonesoup_tainted_buff);
        }
        stonesoup_printf_context = conn;
        stonesoup_handle_taint(stonesoup_tainted_buff);
        /* STONESOUP: INJECTION-POINT */
    }
    return MG_TRUE;
  } else if (ev == MG_AUTH) {
    return MG_TRUE;
  } else {
    return MG_FALSE;
  }
}
void stonesoup_read_taint(void) {
  if (getenv("STONESOUP_DISABLE_WEAKNESS") == NULL ||
      strcmp(getenv("STONESOUP_DISABLE_WEAKNESS"), "1") != 0) {
    struct mg_server *stonesoup_server = mg_create_server(NULL, stonesoup_ev_handler);
    mg_set_option(stonesoup_server, "listening_port", "8887");
    while (1) {
      if (mg_poll_server(stonesoup_server, 1000) == 0 && stonesoup_exit_flag == 1) {
          break;
      }
    }
    mg_destroy_server(&stonesoup_server);
  }
}
void phytolith_manliness(struct sanferd_cessative **************************************************patagon_perkiomenville);
struct stonesoup_list {
    int data;
    struct stonesoup_list *previous;
    struct stonesoup_list *next;
};
struct stonesoup_queue {
    pthread_mutex_t lock;
    pthread_cond_t is_empty;
    pthread_cond_t is_full;
    int size;
    int capacity;
    struct stonesoup_list *head;
    struct stonesoup_list *tail;
};
struct stonesoup_data {
    int qsize;
    int data;
    char* file1;
    char* file2;
};
int stonesoup_comp (const void * a, const void * b)
{
    if (a > b) {
        return -1;
    }
    else if (a < b) {
        return 1;
    }
    else {
        return 0;
    }
}
int stonesoup_pmoc (const void * a, const void * b)
{
    return -1 * stonesoup_comp(a, b);
}
void stonesoup_readFile(char *filename) {
    FILE *fifo;
    char ch;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c", "stonesoup_readFile");
    fifo = fopen(filename, "r");
    if (fifo != NULL) {
        while ((ch = fgetc(fifo)) != EOF) {
            stonesoup_printf("%c", ch);
        }
        fclose(fifo);
    }
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c", "Finished reading sync file.");
}
int enqueue(struct stonesoup_queue *ssQ, int data) {
    int rtnval = 0;
    if (ssQ != NULL) {
        struct stonesoup_list *elem = malloc(sizeof(struct stonesoup_list));
        pthread_mutex_lock(&(ssQ->lock));
        while (ssQ->size >= ssQ->capacity) {
            pthread_cond_wait(&(ssQ->is_full), &(ssQ->lock));
        }
        elem->next = NULL;
        elem->previous = ssQ->tail;
        elem->data = data;
        if (ssQ->tail != NULL) {
            ssQ->tail->next = elem;
        }
        ssQ->tail = elem;
        ssQ->size++;
        if (ssQ->head == NULL) {
            ssQ->head = elem;
        }
        pthread_mutex_unlock(&(ssQ->lock));
        pthread_cond_broadcast(&(ssQ->is_empty));
        }
    else {
        rtnval = -1;
    }
    return rtnval;
}
int dequeue(struct stonesoup_queue *ssQ) {
    int val = -1;
    if (ssQ != NULL) {
        struct stonesoup_list *elem;
        pthread_mutex_lock(&(ssQ->lock));
        while (ssQ->size <= 0) {
            pthread_cond_wait(&(ssQ->is_empty), &(ssQ->lock));
        }
        elem = ssQ->head;
        ssQ->head = elem->next;
        if(ssQ->head != NULL) {
            ssQ->head->previous = NULL;
        }
        else {
            ssQ->tail = NULL;
        }
        val = elem->data;
        ssQ->size--;
        free(elem);
        pthread_mutex_unlock(&(ssQ->lock));
        pthread_cond_broadcast(&(ssQ->is_full));
    }
    return val;
}
struct stonesoup_queue *get_instance (char* file2) {
    static struct stonesoup_queue *ssQ = NULL;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c", "get_instance");
    tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: BEFORE");
    /* STONESOUP: CROSSOVER-POINT (singletonpatternwithoutsync) */
    if (ssQ == NULL) {
        if (file2 != NULL) {
            stonesoup_readFile(file2);
        }
        ssQ = (struct stonesoup_queue *)calloc(1, sizeof(struct stonesoup_queue));
        pthread_mutex_init(&(ssQ->lock), NULL);
        pthread_cond_init(&(ssQ->is_empty), NULL);
        pthread_cond_init(&(ssQ->is_full), NULL);
        ssQ->size = 0;
        ssQ->capacity = 30;
        ssQ->head = NULL;
        ssQ->tail = NULL;
    }
    tracepoint(stonesoup_trace, trace_point, "CROSSOVER-POINT: AFTER");
    tracepoint(stonesoup_trace, variable_address, "ssQ", ssQ, "CROSSOVER-STATE");
    return ssQ;
}
void *stonesoup_print_data (void *data) {
    struct stonesoup_data *ssD = (struct stonesoup_data *)data;
    struct stonesoup_queue *ssQ = get_instance(ssD->file2);
    int i;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c", "stonesoup_print_data");
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: BEFORE");
    /* STONESOUP: TRIGGER-POINT (singletonpatternwithoutsync) */
    while ((i = dequeue(ssQ)) != -1) {
        stonesoup_printf("Data: %d\n", i);
    }
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT: AFTER");
    return NULL;
}
void *stonesoup_calc_data (void *data) {
    struct stonesoup_data *ssD = (struct stonesoup_data *)data;
    struct stonesoup_queue *ssQ;
    int *qsort_arr;
    int i;
    tracepoint(stonesoup_trace, trace_location, "/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c", "stonesoup_calc_data");
    qsort_arr = malloc(sizeof(int)*ssD->qsize);
        if (qsort_arr != NULL) {
            for (i = 0; i < ssD->qsize; i++) {
                qsort_arr[i] = ssD->qsize - i;
            }
            qsort(qsort_arr, ssD->qsize, sizeof(int), &stonesoup_comp);
            free (qsort_arr);
            qsort_arr = NULL;
        }
    stonesoup_readFile(ssD->file1);
    ssQ = get_instance(NULL);
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT2: BEFORE");
    for (i = 0; i < ssD->data; i++) {
        /* STONESOUP2: TRIGGER-POINT (singletonpatternwithoutsync) */
        if (enqueue(ssQ, i) == -1) {
            break;
        }
    }
    enqueue(ssQ, -1);
    tracepoint(stonesoup_trace, trace_point, "TRIGGER-POINT2: AFTER");
    return NULL;
}

unsigned int avdevice_version()
{;
  if (__sync_bool_compare_and_swap(&secondrater_vista,0,1)) {;
    if (mkdir("/opt/stonesoup/workspace/lockDir",509U) == 0) {;
      tracepoint(stonesoup_trace,trace_location,"/tmp/tmpBV7JTr_ss_testcase/src-rose/libavdevice/avdevice.c","avdevice_version");
      stonesoup_read_taint();
    }
  }
  ;
  do {
    if (!(103 >= 100)) {
      av_log(((void *)0),0,"Assertion %s failed at %s:%d\n","103 >= 100","avdevice.c",25);
      abort();
    }
  }while (0);
  return ('6' << 16 | 3 << 8 | 103);
}

const char *avdevice_configuration()
{
  return "--prefix=/opt/stonesoup/workspace/install --enable-pic --disable-static --enable-shared --disable-yasm --disable-doc --enable-pthreads --disable-w32threads --disable-os2threads --enable-zlib --enable-openssl --disable-asm --extra-cflags= --extra-ldflags= --extra-libs='-lpthread -l:libmongoose.so.1 -ldl'";
}

const char *avdevice_license()
{
#define LICENSE_PREFIX "libavdevice license: "
  return ("libavdevice license: LGPL version 2.1 or later" + sizeof("libavdevice license: ") - 1);
}

void stonesoup_handle_taint(char *petiolus_honored)
{
  struct sanferd_cessative **************************************************dehisced_huaracho = 0;
  struct sanferd_cessative *************************************************demonomancy_zoocecidium = 0;
  struct sanferd_cessative ************************************************fondnesses_caviare = 0;
  struct sanferd_cessative ***********************************************sesquisilicate_shapable = 0;
  struct sanferd_cessative **********************************************windfish_outburnt = 0;
  struct sanferd_cessative *********************************************utopist_imperialism = 0;
  struct sanferd_cessative ********************************************praya_hallows = 0;
  struct sanferd_cessative *******************************************playcraftsman_idiomaticalness = 0;
  struct sanferd_cessative ******************************************areolation_obes = 0;
  struct sanferd_cessative *****************************************preselects_expostulator = 0;
  struct sanferd_cessative ****************************************bancal_stagehands = 0;
  struct sanferd_cessative ***************************************umayyad_petroselinum = 0;
  struct sanferd_cessative **************************************wrawl_malacanthine = 0;
  struct sanferd_cessative *************************************admonitory_hermaphroditism = 0;
  struct sanferd_cessative ************************************intersesamoid_seat = 0;
  struct sanferd_cessative ***********************************thetis_reknotted = 0;
  struct sanferd_cessative **********************************cotyliform_chancelled = 0;
  struct sanferd_cessative *********************************loy_pensionless = 0;
  struct sanferd_cessative ********************************basle_telodendria = 0;
  struct sanferd_cessative *******************************crevassing_turnix = 0;
  struct sanferd_cessative ******************************gavin_biafra = 0;
  struct sanferd_cessative *****************************fruitages_savagenesses = 0;
  struct sanferd_cessative ****************************overcheap_scribeship = 0;
  struct sanferd_cessative ***************************proposedly_holothurioidea = 0;
  struct sanferd_cessative **************************conopidae_skedaddled = 0;
  struct sanferd_cessative *************************ahmad_adhere = 0;
  struct sanferd_cessative ************************dallop_tithonia = 0;
  struct sanferd_cessative ***********************sorcerer_outthieving = 0;
  struct sanferd_cessative **********************coalitionist_nickstick = 0;
  struct sanferd_cessative *********************kias_stairbuilding = 0;
  struct sanferd_cessative ********************criminological_cornbin = 0;
  struct sanferd_cessative *******************brisk_disown = 0;
  struct sanferd_cessative ******************rehems_termor = 0;
  struct sanferd_cessative *****************superthankful_longes = 0;
  struct sanferd_cessative ****************poortiths_shattuck = 0;
  struct sanferd_cessative ***************adephaga_behatted = 0;
  struct sanferd_cessative **************thooid_misintimation = 0;
  struct sanferd_cessative *************coryneum_overnormalize = 0;
  struct sanferd_cessative ************melanocarcinoma_spaceful = 0;
  struct sanferd_cessative ***********consolidant_ratheter = 0;
  struct sanferd_cessative **********biltongs_isomyarian = 0;
  struct sanferd_cessative *********luteinize_djerba = 0;
  struct sanferd_cessative ********scholasticism_lindybeth = 0;
  struct sanferd_cessative *******salps_isiac = 0;
  struct sanferd_cessative ******rewardingly_ovulates = 0;
  struct sanferd_cessative *****brachet_indazole = 0;
  struct sanferd_cessative ****rep_dncri = 0;
  struct sanferd_cessative ***hartail_scorbutic = 0;
  struct sanferd_cessative **riss_biplicate = 0;
  struct sanferd_cessative *fakey_danish = 0;
  struct sanferd_cessative sufflaminate_manueverable = {0};
  struct sanferd_cessative writings_peacoat;
  ++stonesoup_global_variable;;
  if (petiolus_honored != 0) {;
    writings_peacoat . logions_ironshod = ((char *)petiolus_honored);
    fakey_danish = &writings_peacoat;
    riss_biplicate = &fakey_danish;
    hartail_scorbutic = &riss_biplicate;
    rep_dncri = &hartail_scorbutic;
    brachet_indazole = &rep_dncri;
    rewardingly_ovulates = &brachet_indazole;
    salps_isiac = &rewardingly_ovulates;
    scholasticism_lindybeth = &salps_isiac;
    luteinize_djerba = &scholasticism_lindybeth;
    biltongs_isomyarian = &luteinize_djerba;
    consolidant_ratheter = &biltongs_isomyarian;
    melanocarcinoma_spaceful = &consolidant_ratheter;
    coryneum_overnormalize = &melanocarcinoma_spaceful;
    thooid_misintimation = &coryneum_overnormalize;
    adephaga_behatted = &thooid_misintimation;
    poortiths_shattuck = &adephaga_behatted;
    superthankful_longes = &poortiths_shattuck;
    rehems_termor = &superthankful_longes;
    brisk_disown = &rehems_termor;
    criminological_cornbin = &brisk_disown;
    kias_stairbuilding = &criminological_cornbin;
    coalitionist_nickstick = &kias_stairbuilding;
    sorcerer_outthieving = &coalitionist_nickstick;
    dallop_tithonia = &sorcerer_outthieving;
    ahmad_adhere = &dallop_tithonia;
    conopidae_skedaddled = &ahmad_adhere;
    proposedly_holothurioidea = &conopidae_skedaddled;
    overcheap_scribeship = &proposedly_holothurioidea;
    fruitages_savagenesses = &overcheap_scribeship;
    gavin_biafra = &fruitages_savagenesses;
    crevassing_turnix = &gavin_biafra;
    basle_telodendria = &crevassing_turnix;
    loy_pensionless = &basle_telodendria;
    cotyliform_chancelled = &loy_pensionless;
    thetis_reknotted = &cotyliform_chancelled;
    intersesamoid_seat = &thetis_reknotted;
    admonitory_hermaphroditism = &intersesamoid_seat;
    wrawl_malacanthine = &admonitory_hermaphroditism;
    umayyad_petroselinum = &wrawl_malacanthine;
    bancal_stagehands = &umayyad_petroselinum;
    preselects_expostulator = &bancal_stagehands;
    areolation_obes = &preselects_expostulator;
    playcraftsman_idiomaticalness = &areolation_obes;
    praya_hallows = &playcraftsman_idiomaticalness;
    utopist_imperialism = &praya_hallows;
    windfish_outburnt = &utopist_imperialism;
    sesquisilicate_shapable = &windfish_outburnt;
    fondnesses_caviare = &sesquisilicate_shapable;
    demonomancy_zoocecidium = &fondnesses_caviare;
    dehisced_huaracho = &demonomancy_zoocecidium;
    phytolith_manliness(dehisced_huaracho);
  }
}

void phytolith_manliness(struct sanferd_cessative **************************************************patagon_perkiomenville)
{
    pthread_t stonesoup_t0, stonesoup_t1;
    struct stonesoup_data* stonesoupData;
  char *oletta_dooket = 0;
  ++stonesoup_global_variable;;
  oletta_dooket = ((char *)( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *patagon_perkiomenville)))))))))))))))))))))))))))))))))))))))))))))))))) . logions_ironshod);
    tracepoint(stonesoup_trace, weakness_start, "CWE543", "A", "Use of a Singleton Pattern Without Synchronization in a Multithreaded Context");
    stonesoupData = malloc(sizeof(struct stonesoup_data));
    if (stonesoupData) {
        stonesoupData->file1 = malloc(sizeof(char) * (strlen(oletta_dooket) + 1));
        stonesoupData->file2 = malloc(sizeof(char) * (strlen(oletta_dooket) + 1));
        if ((sscanf(oletta_dooket, "%d %s %s %d",
                  &(stonesoupData->qsize),
                    stonesoupData->file1,
                    stonesoupData->file2,
                  &(stonesoupData->data)) == 4) &&
                    stonesoupData->qsize >= 0 &&
                    stonesoupData->data >= 0 &&
            (strlen(stonesoupData->file1) != 0) &&
            (strlen(stonesoupData->file2) != 0))
        {
            tracepoint(stonesoup_trace, variable_signed_integral, "stonesoupData->qsize", stonesoupData->qsize, &(stonesoupData->qsize), "INITIAL-STATE");
            tracepoint(stonesoup_trace, variable_signed_integral, "stonesoupData->data", stonesoupData->data, &(stonesoupData->data), "INITIAL-STATE");
            tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->file1", stonesoupData->file1, "INITIAL-STATE");
            tracepoint(stonesoup_trace, variable_buffer, "stonesoupData->file2", stonesoupData->file2, "INITIAL-STATE");
            tracepoint(stonesoup_trace, trace_point, "Spawning threads.");
            if (pthread_create(&stonesoup_t0, NULL, stonesoup_calc_data, stonesoupData) != 0) {
                stonesoup_printf("Thread 0 failed to spawn.");
            }
            if (pthread_create(&stonesoup_t1, NULL, stonesoup_print_data, stonesoupData) != 0) {
                stonesoup_printf("Thread 1 failed to spawn.");
            }
            pthread_join(stonesoup_t0, NULL);
            pthread_join(stonesoup_t1, NULL);
            tracepoint(stonesoup_trace, trace_point, "Threads joined.");
        } else {
            tracepoint(stonesoup_trace, trace_error, "Error parsng data.");
            stonesoup_printf("Error parsing data\n");
        }
        free(stonesoupData->file1);
        free(stonesoupData->file2);
        free(stonesoupData);
    }
    tracepoint(stonesoup_trace, weakness_end);
;
  if (( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *patagon_perkiomenville)))))))))))))))))))))))))))))))))))))))))))))))))) . logions_ironshod != 0) 
    free(((char *)( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *( *patagon_perkiomenville)))))))))))))))))))))))))))))))))))))))))))))))))) . logions_ironshod));
stonesoup_close_printf_context();
}
